
/*------------------------------------------------------------------------
   File        : ImageUtil
   Purpose     : 
   Syntax      : 
   Description : 
   Author(s)   : tom.bergman
   Created     : Mon Dec 12 14:01:18 EST 2011
   Notes       : 
 ----------------------------------------------------------------------*/

USING Progress.Lang.*.
USING System.IO.*.
USING System.Drawing.*.
USING System.Drawing.Imaging.*.
USING System.Drawing.Drawing2D.*.
USING System.Windows.Forms.*.

CLASS ImageUtil: 
    
  /*------------------------------------------------------------------------------
   Purpose:
   Notes:
  ------------------------------------------------------------------------------*/
  DEFINE VARIABLE oConverter AS ImageConverter.	
	
  CONSTRUCTOR PUBLIC ImageUtil ():
    SUPER ().
    oConverter = NEW ImageConverter().
  END CONSTRUCTOR.


  METHOD PUBLIC Bitmap CaptureForm( pForm AS Progress.Windows.Form ):
    
    RETURN CaptureScreen(
      pForm:DesktopBounds:X, 
      pForm:DesktopBounds:Y, 
      pForm:DesktopBounds:Width, 
      pForm:DesktopBounds:Height).
    
  END METHOD.

  METHOD PUBLIC Bitmap CaptureScreen(  ):
        
    RETURN CaptureScreen(
      Screen:PrimaryScreen:Bounds:X,
      Screen:PrimaryScreen:Bounds:Y,
      Screen:PrimaryScreen:Bounds:Width,
      Screen:PrimaryScreen:Bounds:Height).

  END METHOD.
  
  METHOD PUBLIC Bitmap CaptureScreen( x AS INTEGER, y AS INTEGER, width AS INTEGER, height AS INTEGER ):
    
    DEFINE VARIABLE oBitmap   AS Bitmap.
    DEFINE VARIABLE oGraphics AS Graphics.

    oBitmap = NEW Bitmap(Width, Height).
    oGraphics = Graphics:FromImage(oBitmap).
    
    oGraphics:CopyFromScreen(
      X,
      Y,
      0,
      0,
      oBitmap:Size,
      CopyPixelOperation:SourceCopy).
    oGraphics:Dispose().
    RETURN oBitmap.
  
  END METHOD.

  METHOD PUBLIC Bitmap CaptureWindow( winHandle AS HANDLE ):
    
    DEFINE VARIABLE vWidth    AS INTEGER.
    DEFINE VARIABLE vHeight   AS INTEGER.
    DEFINE VARIABLE lpRect    AS MEMPTR.
    DEFINE VARIABLE iResult   AS INTEGER.
    DEFINE VARIABLE parentHdl AS INTEGER.
    set-size(lpRect) = 16.
    
    RUN GetParent(winHandle:HWND,
      OUTPUT parentHdl).
        
    RUN GetWindowRect(
      INPUT parentHdl,
      OUTPUT lpRect,
      OUTPUT iResult
      ).
    vWidth  = GET-LONG(lpRect, 9)  - GET-LONG(lpRect, 1).
    vHeight = GET-LONG(lpRect, 13) - GET-LONG(lpRect, 5).
    set-size(lpRect) = 0.
        
    RETURN captureScreen(winhandle:X, winHandle:Y, vWidth, vHeight).

  END METHOD.

  METHOD PRIVATE ImageCodecInfo GetEncoderInfo( MimeType AS CHARACTER ):
    DEFINE VARIABLE i     AS INTEGER.
    DEFINE VARIABLE Codec AS ImageCodecInfo EXTENT.
    
    Codec = ImageCodecInfo:GetImageEncoders().
    
    DO i = 1 TO EXTENT(Codec):
      IF Codec[i]:MimeType = MimeType THEN
        RETURN Codec[i].

    END.
    RETURN ?.

  END METHOD.

  METHOD PUBLIC CHARACTER getImageFormat( INPUT pImage AS Bitmap ):
    IF ImageFormat:Jpeg:Equals(pImage:RawFormat) 
      THEN RETURN "jpeg".
    IF ImageFormat:Bmp:Equals(pImage:RawFormat) 
      THEN RETURN "bmp".
    IF ImageFormat:Emf:Equals(pImage:RawFormat) 
      THEN RETURN "emf".
    IF ImageFormat:Exif:Equals(pImage:RawFormat) 
      THEN RETURN "exif".
    IF ImageFormat:Gif:Equals(pImage:RawFormat) 
      THEN RETURN "gif".
    IF ImageFormat:Icon:Equals(pImage:RawFormat) 
      THEN RETURN "icon".
    IF ImageFormat:Png:Equals(pImage:RawFormat) 
      THEN RETURN "png".
    IF ImageFormat:Tiff:Equals(pImage:RawFormat) 
      THEN RETURN "tiff".
    IF ImageFormat:Wmf:Equals(pImage:RawFormat) 
      THEN RETURN "wmf".
    RETURN "".
  END METHOD.

  METHOD PUBLIC Bitmap imageFromByteArray( oByte AS "System.Byte[]" ):
    
    RETURN CAST(oConverter:ConvertFrom(oByte),Bitmap).
  END METHOD.

  METHOD PUBLIC Bitmap imageFromMemptr( pMem AS MEMPTR ):
    RETURN imageFromByteArray(PugUtils:MemptrToByteArray(pMem)).
  END METHOD.
  /* The only reason to use this method is if you have just scaled or otherwise changed
     an image and now want to save it to the DB. 
     If your images are files, use Copy-lob to put them in the db, not loadImage and then this 
     unless scaling, resizing or if conversion to jpeg is your goal */
  METHOD PUBLIC VOID ImageToBuffer ( INPUT oImage AS Bitmap, pBuff AS HANDLE, pFieldName AS CHARACTER, pImageMimeType AS CHARACTER):
    DEFINE VARIABLE QualityParameter AS EncoderParameter.
    DEFINE VARIABLE Codec            AS ImageCodecInfo.
    DEFINE VARIABLE EncoderParams    AS EncoderParameters.
    DEFINE VARIABLE MS               AS MemoryStream.
    DEFINE VARIABLE vMem             AS MEMPTR.
    
    QualityParameter = NEW EncoderParameter(Encoder:Quality, 100).
    Codec = GetEncoderInfo(pImageMimeType).
    EncoderParams = NEW EncoderParameters(1).
    EncoderParams:Param:setValue(QualityParameter,0).
    MS = NEW MemoryStream().
    oImage:Save(MS,Codec,EncoderParams).

    vMem = PugUtils:ByteArrayToMemptr(MS:ToArray()).
    MS:Close().

    COPY-LOB vMem TO pBuff:BUFFER-FIELD(pFieldName):Buffer-value.
    set-size(vMem) = 0.
    RETURN.
  END.
  
  /* The only reason to use this method is if you have just scaled or otherwise changed
     an image and now want to save it to the DB. 
     If your images are files, use Copy-lob to put them in the db, not loadImage and then this 
     unless scaling, resizing or if conversion is your goal */
  METHOD PUBLIC VOID ImageToBuffer ( INPUT oImage AS Bitmap, pBuff AS HANDLE, pFieldName AS CHARACTER):
    DEFINE VARIABLE vMem             AS MEMPTR.
        
    vMem = PUGUtils:ByteArrayToMemptr(ImageToByteArray(oImage)).
    COPY-LOB vMem TO pBuff:BUFFER-FIELD(pFieldName):Buffer-value.
    set-size(vMem) = 0. 
   
  END METHOD.

  /* There are a lot of ways to get an image file into a System.Drawing.Bitmap.
     Here are a few:
     1) myBitmap = new Bitmap(<filename>).
        This works but will lock the file for the life of the image.
     2) myBitmap = new Bitmap(<filename>).
        myBitmap2 = new Bitmap(MyBitmap).
        myBitmap:Dispose().
        This cures the locking problem. It's a little slow and most of the PPI and color depth
        attributes of the image are lost. This doesn't usually matter.
     3) myByteArray = System.Io.File:ReadAllBytes(<Filename>).
        myMemStream = new MemoryStream(myByteArray).
        myBitmap = CAST(Image:FromStream(myMemStream),Bitmap).  
        This works, and does not lock but the memorystream must exist for the life of the image.
        If the memorystream gets garbage collected, there will be errors (GDI) at some point 
        if the image is still in use.
     4) The method below seems to be the fastest and retains as much image information as I've 
        found possible. Still wish there was something better.  */        
  
  METHOD PUBLIC Bitmap LoadImage( pFileName AS CHARACTER ):
    RETURN CAST(oConverter:ConvertFrom(System.IO.File:ReadAllBytes(pFileName)),Bitmap).
                                                                           
  END METHOD.

  /*  METHOD PUBLIC Bitmap ImageFromBuffer( pBuff AS HANDLE, pFieldName AS CHARACTER ):*/
  /*                                                                                                  */
  /*    DEFINE VARIABLE MS AS System.IO.MemoryStream      NO-UNDO.                                    */
  /*    DEFINE VARIABLE BS AS Progress.Data.BindingSource NO-UNDO.                                    */
  /*                                                                                                  */
  /*    BS = NEW Progress.Data.BindingSource(pBuff).                                                  */
  /*    BS:Refresh().                                                                                 */
  /*    MS = NEW System.IO.MemoryStream(CAST(BS:InputValue[pFieldName],"System.Byte[]")).             */
  /*    RETURN NEW Bitmap(MS).                                                         */
  /*                                                                                                  */
  /*    FINALLY:                                                                                      */
  /*      BS:Dispose().                                                                               */
  /*    END.                                                                                          */
  /*  END METHOD.                                                                                     */
  METHOD PUBLIC Bitmap ImageFromBuffer( pBuff AS HANDLE, pFieldName AS CHARACTER ):
  
    DEFINE VARIABLE vMem    AS MEMPTR NO-UNDO.
    DEFINE VARIABLE oBitmap AS Bitmap.
    
    COPY-LOB pBuff:BUFFER-FIELD(pFieldName):buffer-value TO vMem.
    oBitmap = ImageFromMemptr(vMem).
    SET-SIZE(vMem) = 0. 
    RETURN oBitmap.
    
  END METHOD.
   
  METHOD PUBLIC "System.Byte[]" ImageToByteArray( pImage AS Bitmap ):
    RETURN CAST(oConverter:ConvertTo(pImage,Progress.Util.TypeHelper:GetType("System.Byte[]")),"System.Byte[]").
  END METHOD.
    
  METHOD PUBLIC Bitmap ResizeImage( pImage AS Bitmap,pWidth AS INTEGER, pHeight AS INTEGER ):
    DEFINE VARIABLE oImage    AS Bitmap.
    DEFINE VARIABLE pGraphics AS Graphics.
    
    oImage = NEW Bitmap(pWidth,pHeight).
    
    pGraphics = Graphics:FromImage(oImage).
    
    pgraphics:CompositingQuality = CompositingQuality:HighQuality. 
    pgraphics:InterpolationMode = InterpolationMode:HighQualityBicubic.
    pgraphics:SmoothingMode = SmoothingMode:HighQuality.
    pgraphics:DrawImage(pImage, 0, 0, oImage:Width, oImage:Height).         
    
    RETURN oImage.

  END METHOD.
  
  /* This method allow specifying the jpeg quality and wraps this complexity.
     All other image types should just use the built in myImage:Save method 
     with the ImageFormat as the second paramemeter as in
     myImage:Save(filename, ImageFormat:Png) */
  METHOD PUBLIC VOID SaveAsJpeg( pPath AS CHARACTER, pImage AS Bitmap, pQuality AS INTEGER ):
        
    DEFINE VARIABLE QualityParameter AS EncoderParameter.
    DEFINE VARIABLE jPegCodec        AS ImageCodecInfo.
    DEFINE VARIABLE EncoderParams    AS EncoderParameters.
    
    QualityParameter = NEW EncoderParameter(Encoder:Quality, pQuality).
    jPegCodec = GetEncoderInfo("image/jpeg").
    EncoderParams = NEW EncoderParameters(1).
    EncoderParams:Param:setValue(QualityParameter,0).
    pImage:Save(pPath,jPegCodec,EncoderParams).
    
  END METHOD.
  
  METHOD PUBLIC Bitmap ScaleImage( pImage AS Bitmap,maxSize AS INTEGER ):
    DEFINE VARIABLE CurrentMax  AS INTEGER NO-UNDO.
    DEFINE VARIABLE scaleAmount AS DECIMAL NO-UNDO.
    
    currentMax = max(pImage:height,pImage:Width).
    scaleAmount = maxSize / currentMax.
    
    RETURN resizeImage(pImage,INT(pImage:WIDTH * scaleAmount),INT(pImage:HEIGHT * scaleAmount)).
    
  END METHOD.
  /* Will only reduce size, never increase */
  METHOD PUBLIC Bitmap ScaleImageDown( pImage AS System.Drawing.Bitmap,maxSize AS INTEGER ):
    DEFINE VARIABLE CurrentMax  AS INTEGER NO-UNDO.
    DEFINE VARIABLE scaleAmount AS DECIMAL NO-UNDO.
    
    currentMax = max(pImage:height,pImage:Width).
    IF currentMax LE maxSize THEN RETURN CAST(pImage,Bitmap).
    scaleAmount = maxSize / currentMax.
    
    RETURN resizeImage(pImage,INT(pImage:WIDTH * scaleAmount),INT(pImage:HEIGHT * scaleAmount)).
    
  END METHOD.
  
PROCEDURE GetWindowRect EXTERNAL "USER32.DLL":
  DEFINE INPUT  PARAMETER hWnd    AS LONG.
  DEFINE OUTPUT PARAMETER lpRect  AS MEMPTR.
  DEFINE RETURN PARAMETER iResult  AS LONG.
END.
PROCEDURE GetParent EXTERNAL "USER32.DLL":
  DEFINE INPUT  PARAMETER plHWND    AS LONG NO-UNDO.
  DEFINE RETURN PARAMETER plWinLong AS LONG NO-UNDO.
END PROCEDURE.


END CLASS.