 
 /*------------------------------------------------------------------------
    File        : PUGUtils
    Purpose     : 
    Syntax      : 
    Description : 
    Author(s)   : Tom.Bergman
    Created     : Tue Mar 11 10:34:56 EDT 2014
    Notes       : 
  ----------------------------------------------------------------------*/

USING Progress.Lang.*. 

CLASS PUGUtils: 

	METHOD PUBLIC STATIC MEMPTR ByteArrayToMemptr( nBytes AS "System.Byte[]" ):
    DEFINE VARIABLE nPtr AS System.IntPtr NO-UNDO.
    DEFINE VARIABLE mPtr AS MEMPTR        NO-UNDO.

    set-size(mPtr) = nBytes:LENGTH.
    nPtr = NEW System.IntPtr(GET-POINTER-VALUE(mPtr)).
    System.Runtime.InteropServices.Marshal:Copy(nBytes, 0, nPtr, nBytes:LENGTH).
    RETURN mPtr.
    FINALLY:
      nPtr = ?.
      nBytes = ?. 
      set-size(mPtr) = 0.
    END.
  END METHOD.
  
  /*    Be careful not to reuse the ByteArray you're passing in. Code like:
        myByte = winOS:CompressByteArray(myByte)
        will lead to corrupt data as you can't simply reset a ByteArray 
        as it already has content */
  METHOD PUBLIC STATIC "System.Byte[]" CompressByteArray( inByte AS "System.Byte[]" ):
    DEFINE VARIABLE OutMemStream   AS System.IO.MemoryStream.
    DEFINE VARIABLE CompressStream AS System.IO.Compression.GZipStream.
    OutMemStream = NEW System.IO.MemoryStream().
    CompressStream = NEW System.IO.Compression.GZipStream(OutMemStream,System.IO.Compression.CompressionMode:Compress).
    CompressStream:Write(inByte,0,inByte:Length).
    /* Without the close, the stream may not finish writing */
    CompressStream:Close().
    RETURN OutMemStream:ToArray().
    FINALLY:
      OutMemStream:Close().
      inByte = ?.
    END.
    
  END METHOD.
  
  METHOD PUBLIC STATIC MEMPTR CompressMemptr( inMemptr AS MEMPTR ):
    RETURN ByteArrayToMemPtr(CompressByteArray(MemptrToByteArray(inMemptr))).
    FINALLY:
      set-size(inMemptr) = 0.
    END.    
  END METHOD.
  
  METHOD PUBLIC STATIC "System.Byte[]" DecompressByteArray( pByte AS "System.Byte[]" ):
    DEFINE VARIABLE MS            AS System.IO.MemoryStream.
    DEFINE VARIABLE DecompressOut AS System.IO.Compression.GZipStream.
    DEFINE VARIABLE outMem        AS System.IO.MemoryStream. 
        
    MS = NEW System.IO.MemoryStream(pByte).
    DecompressOut = NEW System.IO.Compression.GZipStream(MS,System.IO.Compression.CompressionMode:Decompress).
    OutMem = NEW System.IO.MemoryStream().
    DecompressOut:CopyTo(OutMem).
    /* Without the close, the stream may not finish writing */
    DecompressOut:Close().
    MS:Close().
    RETURN OutMem:ToArray().
    FINALLY:
      OutMem:Close().
      pByte = ?.
    END.
  END METHOD.
    
  METHOD PUBLIC STATIC MEMPTR DeCompressMemptr( inMemptr AS MEMPTR ):
    RETURN ByteArrayToMemPtr(DecompressByteArray(MemptrToByteArray(inMemptr))).
    FINALLY:
      set-size(inMemptr) = 0.
    END.        
  END METHOD.
  
  /*------------------------------------------------------------------------------
   Needs System.Xml.Linq in assembly file
  
   Notes: Due to a Progress bug, this is needlessly complicated.
   The Parse method of the xdoc fails when the longchar contains utf-8 
   characters so we're converting the longchar to a Memptr, 
   the Memptr to a byte array, the byte array to a memory stream,
    and then using the load method on the xDoc.
   
   http://knowledgebase.progress.com/articles/Article/000050259?popup=true
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC STATIC LONGCHAR FormatXml( pXml AS LONGCHAR ):
    DEFINE VARIABLE myByte AS "System.Byte[]".
    DEFINE VARIABLE ms     AS System.IO.MemoryStream.
    DEFINE VARIABLE vMem   AS MEMPTR.
    DEFINE VARIABLE xdoc   AS System.Xml.Linq.XDocument.
    DEFINE VARIABLE vLong  AS LONGCHAR                  NO-UNDO.
    
    COPY-LOB pXml TO vMem NO-CONVERT.
    myByte = MemPtrToByteArray(vMem).
    set-size(vMem) = 0.
    ms = NEW System.IO.MemoryStream(myByte).
    xdoc = System.Xml.Linq.XDocument:Load(ms).
    ms:Close().
    /* A plain return of the ToString won't always be seen as a longchar 
   in copy-lob even though the return type is longchar */
    vLong = xdoc:ToString().
    RETURN vLong.
  END METHOD.
  
  
  METHOD PUBLIC STATIC "System.Byte[]" MemptrToByteArray( pmptr AS MEMPTR ):
    DEFINE VARIABLE nPtr   AS System.IntPtr   NO-UNDO.
    DEFINE VARIABLE vInt   AS INTEGER         NO-UNDO.
    DEFINE VARIABLE nBytes AS "System.Byte[]".
    
    vInt = GET-SIZE(pmPtr).
    nBytes = NEW "System.Byte[]"(vInt).
    nPtr = NEW System.IntPtr(GET-POINTER-VALUE(pmPtr)).
    System.Runtime.InteropServices.Marshal:Copy(nPtr, nBytes, 0, vInt).
    
    RETURN nBytes.
    FINALLY:
      nPtr = ?.
      set-size(pmPtr) = 0.
      nBytes = ?.
    END.
        
  END METHOD.
  
  

END CLASS.