
/*------------------------------------------------------------------------
    File        : ValidateEmail.p
    Purpose     : 

    Syntax      :

    Description : 

    Author(s)   : 
    Created     :  Wed May 13 13:12:01 EDT 2015
    Notes       :  The mailaddress method is very generous in passing making
                   it almost useless. It will accept Bill.Gates@Microsoft as 
                   valid. It does, however, do an excellent job of parsing out 
                   the pieces of an address.
                   The EmailAddressAttribute does a much better job but only
                   on a basic address, not one with a display name.
                   The combination of the two works well.
  ----------------------------------------------------------------------*/
DEFINE VARIABLE email                  AS CHARACTER.
DEFINE VARIABLE oMailAddress           AS System.Net.Mail.MailAddress.
DEFINE VARIABLE oEmailAddressAttribute AS System.ComponentModel.DataAnnotations.EmailAddressAttribute.

/* MailAddress will detect this as bad because of the colon*/
email = "<Bill Gates> Bill:Gates@Microsoft.com".
/* Only EmailAddressAttribute will detect this as bad because of missing domain */
email = "<Bill Gates> Bill.Gates@Microsoft".
/* This is a good address being parsed by EmailAddressAttribute */
email = "<Bill Gates> Bill.Gates@Microsoft.com".

oMailAddress =  NEW System.Net.Mail.MailAddress(eMail) NO-ERROR.

IF ERROR-STATUS:ERROR THEN
  MESSAGE "Bad Address from MailAddress"
    VIEW-AS ALERT-BOX.
ELSE 
DO:
  oEmailAddressAttribute = 
    NEW System.ComponentModel.DataAnnotations.EmailAddressAttribute().
  IF oEmailAddressAttribute:IsValid(oMailAddress:Address) THEN 
    MESSAGE "Good Address" SKIP
      "Display Name: " oMailAddress:DisplayName SKIP
      "Email Address:" oMailAddress:Address SKIP 
      "User: " oMailAddress:User SKIP
      "Host: " oMailAddress:Host VIEW-AS ALERT-BOX.
  ELSE 
    MESSAGE "Bad Address from EmailAddressAttribute"
      VIEW-AS ALERT-BOX.  
END.

  