
USING System.Net.*.
USING System.Xml.*.

DEFINE VARIABLE Client      AS WebClient.
DEFINE VARIABLE requestXML  AS CHARACTER.
DEFINE VARIABLE xDoc        AS XmlDocument.
DEFINE VARIABLE nsManager   AS XmlNamespaceManager.
DEFINE VARIABLE ErrResponse AS LONGCHAR.
DEFINE VARIABLE ZipCode     AS CHARACTER.

zipcode = "90210".  

/* This xml was determined by using SoapUI to read the WSDL and 
   then access this service */ 
requestXML =      
  '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:weat="http://ws.cdyne.com/WeatherWS/">' +
  '  <soapenv:Header/>' +
  '  <soapenv:Body>' +
  '    <weat:GetCityWeatherByZIP>' +
  '      <!--Optional:-->' +
  '      <weat:ZIP>' + zipcode + '</weat:ZIP>' +
  '    </weat:GetCityWeatherByZIP>' +
  '  </soapenv:Body>' +
  '</soapenv:Envelope>'.  

Client = NEW WebClient().

Client:Headers:Add("Content-Type", "text/xml;charset=UTF-8").
/* This particular SOAP service needs this header */
Client:Headers:Add("SOAPAction","http://ws.cdyne.com/WeatherWS/GetCityWeatherByZIP").
/* This is not strictly needed, this URL could be the first
   parameter of the UploadString method */
Client:BaseAddress = "http://wsf.cdyne.com/WeatherWS/Weather.asmx".

/* We could use any of the Progress xml capabilities, but since
   this is a .Net demo .... */
xDoc = NEW XmlDocument().
/* UploadString lets me do a POST. It's the default but shown for clarity.
   Leaving out the second parameter would behave the same. */

xDoc:LoadXml(Client:UploadString("","POST",requestXML)).

/* This XML has namespaces. To get to the nodes, we need to tell
   the xdoc object about the namespace containing the nodes we
   want to retrieve */
nsManager = NEW XmlNamespaceManager(xDoc:NameTable).
/* Add this namespace, name it wws. It does not already 
   have a namespace prefix. */
nsManager:AddNamespace("wws", "http://ws.cdyne.com/WeatherWS/").

MESSAGE
  STRING(PUGUtils:FormatXml(xDoc:InnerXml)) SKIP(1)
  VIEW-AS ALERT-BOX.
 
MESSAGE
  "City: " xdoc:SelectSingleNode("//wws:City", nsManager):InnerText SKIP
  "Description: " xdoc:SelectSingleNode("//wws:Description", nsManager):InnerText SKIP
  "Temperature: " xdoc:SelectSingleNode("//wws:Temperature", nsManager):InnerText SKIP
  "WindSpeed: " xdoc:SelectSingleNode("//wws:Wind", nsManager):InnerText SKIP
  "Pressure:" xdoc:SelectSingleNode("//wws:Pressure", nsManager):InnerText 
  VIEW-AS ALERT-BOX.
  
CATCH e AS System.Net.WebException:
  MESSAGE
    "Status: " e:Status SKIP
    "Message: " e:Message SKIP
    "GetMessage(1): " e:getMessage(1)
    VIEW-AS ALERT-BOX.
  
  /* When WebClient gets an http error (non 200) it raises an exception.
     There will be the usual messages in the exception but the content,
     which might include the SOAP fault does not get returned in the 
     usual way. Instead, there will be a WebResponse object on the exception
     that can be processed to get the data. */
  IF TYPE-OF(e:Response,HttpWebResponse) THEN 
  DO:
    DEFINE VARIABLE HttpResponse AS HttpWebResponse.
    /* The HttpWebResponse object has StatusCode and other properties
       not present in WebResponse */
    HttpResponse = CAST(e:Response, HttpWebResponse).
    xDoc = NEW XmlDocument().
    xDoc:Load(HttpResponse:GetResponseStream()).
    nsManager = NEW XmlNamespaceManager(xDoc:NameTable).
    nsManager:AddNamespace("soap", "http://schemas.xmlsoap.org/soap/envelope/").
    
    MESSAGE
      STRING(PugUtils:FormatXml(xDoc:OuterXml)) SKIP(2)
      xDoc:SelectSingleNode("//faultstring", nsManager):InnerText SKIP (2)
      HttpResponse:StatusCode:value__
      VIEW-AS ALERT-BOX.
  END.  
END CATCH.


  