
USING System.Net.Http.*.
USING System.Xml.*.
USING System.Text.*.

DEFINE VARIABLE Client          AS HttpClient.
DEFINE VARIABLE ResponseMessage AS HttpResponseMessage.
DEFINE VARIABLE RequestMessage  AS HttpRequestMessage.
DEFINE VARIABLE RequestXml      AS CHARACTER.
DEFINE VARIABLE xDoc            AS XmlDocument.
DEFINE VARIABLE nsManager       AS XmlNamespaceManager.
DEFINE VARIABLE StockSym        AS CHARACTER.

StockSym = "PRGS".  

/* This xml was determined by using SoapUI to read the WSDL and 
   then access this service */ 

RequestXml =
  "<soapenv:Envelope xmlns:soapenv=~"http://schemas.xmlsoap.org/soap/envelope/~" xmlns:ws=~"http://ws.cdyne.com/~">" +
  "   <soapenv:Header/>" +
  "   <soapenv:Body>" +
  "      <ws:GetQuote>" +
  "         <!--Optional:-->" +
  "         <ws:StockSymbol>" + StockSym + "</ws:StockSymbol>" +
  "         <!--Optional:-->" +
  "         <ws:LicenseKey>0</ws:LicenseKey>" +
  "      </ws:GetQuote>" +
  "   </soapenv:Body>" +
  "</soapenv:Envelope>" .
  
  

/* By using RequestMessage and ResponseMessage objects, we can get 
   somewhat finer grained control. This could have all been done
   with a Client:PostAsync call which also returns a ResponseMessage. */
RequestMessage = NEW HttpRequestMessage().
/* Creating a stringContent object with this constructor will
   automatically set the HTTP Content-Type and Encoding */
RequestMessage:Content = 
  NEW StringContent(RequestXml, Encoding:UTF8, "text/xml").
/* The default is Get, custom methods are possible with a Syntax like */
/* RequestMessage:Method = NEW HttpMethod("PATCH").*/
RequestMessage:Method = HttpMethod:Post.

Client = NEW HttpClient().
Client:BaseAddress = NEW System.Uri("http://ws.cdyne.com/delayedstockquote/delayedstockquote.asmx").

ResponseMessage = Client:SendAsync(RequestMessage):Result.


/* We could use any of the Progress xml capabilities, but since
   this is a .Net demo .... */  
xDoc = NEW XmlDocument().
/* The ResponseMessage has 3 basic payloads. We can get a string, a 
   bytearray, or a stream depending upon the ReadAs... method
   used. In this case we'll use a stream as the XmlDocument object
   can load that using the Load method. There's also a LoadXml method
   that could have loaded the String version of the data */
xDoc:Load(ResponseMessage:Content:ReadAsStreamAsync():Result).

/* This XML has namespaces. To get to the nodes, we need to tell
   the xdoc object about the namespace containing the nodes we
   want to retrieve */
nsManager = NEW XmlNamespaceManager(xDoc:NameTable).
/* Add this namespace, name it cdyne. It does not already 
   have a namespace prefix. */
nsManager:AddNamespace("cdyne", "http://ws.cdyne.com/").

MESSAGE
  STRING(PUGUtils:FormatXml(xDoc:InnerXml)) SKIP(1)
  VIEW-AS ALERT-BOX.
 
MESSAGE
  "Last Trade:" xdoc:SelectSingleNode("//cdyne:LastTradeDateTime", nsManager):InnerText SKIP
  "Open: " xdoc:SelectSingleNode("//cdyne:OpenAmount", nsManager):InnerText SKIP
  "Day High: " xdoc:SelectSingleNode("//cdyne:DayHigh", nsManager):InnerText SKIP
  "Day Low: " xdoc:SelectSingleNode("//cdyne:DayLow", nsManager):InnerText SKIP
  "Volume: " xdoc:SelectSingleNode("//cdyne:StockVolume", nsManager):InnerText SKIP
  "Change: " xdoc:SelectSingleNode("//cdyne:ChangePercent", nsManager):InnerText SKIP
  "52 Week Range: " xdoc:SelectSingleNode("//cdyne:FiftyTwoWeekRange", nsManager):InnerText SKIP
    
  VIEW-AS ALERT-BOX.
