USING OpenEdge.Net.*.
USING OpenEdge.Net.HTTP.*. 
USING Progress.Json.ObjectModel.*.

DEFINE VARIABLE i                       AS INTEGER.
DEFINE VARIABLE oRequest                AS IHttpRequest.
DEFINE VARIABLE oResponse               AS IHttpResponse.
DEFINE VARIABLE oClient                 AS IHttpClient.
DEFINE VARIABLE oJsonResponseObject     AS JsonObject.
DEFINE VARIABLE oWeatherObject          AS JsonObject.
DEFINE VARIABLE oCurrentConditionObject AS JsonObject.
DEFINE VARIABLE oWeatherArray           AS JsonArray.
DEFINE VARIABLE oHourlyArray            AS JsonArray.
DEFINE VARIABLE oUri                    AS Uri.
DEFINE VARIABLE ProgressDS              AS HANDLE.
DEFINE VARIABLE hBuff                   AS HANDLE.
DEFINE VARIABLE y                       AS INTEGER.
DEFINE VARIABLE FieldList               AS CHARACTER.

/*vUrl = "http://api.worldweatheronline.com/premium/v1/weather.ashx?key=TrialKey&format=json&num_of_days=5".*/

oUri = NEW Uri("api.worldweatheronline.com").
oUri:Path = "/premium/v1/weather.ashx".
oUri:AddQuery("key","NeedTrialKeyHere").
oUri:AddQuery("format","json").
oUri:AddQuery("num_of_days","5").
oUri:AddQuery("q", "01821").

oRequest = RequestBuilder:Get(oUri):Request.

/*oClient = ClientBuilder:Build():Client.*/
/*This mess uses .Net to figure out the Proxy (if any) and automatically
  use it. This enables fiddler and I expect, other proxies but since it
  uses .Net, is windows only. */
oClient =
  ClientBuilder
    :Build()
    :ViaProxy(
  System.Net.WebRequest:GetSystemWebProxy()
      :GetProxy(NEW System.Uri(oUri:BaseUri))
      :AbsoluteUri)
    :Client.

oResponse = oClient:Execute(oRequest).

oJsonResponseObject = CAST(oResponse:Entity, JsonObject).

oCurrentConditionObject = 
  /* The root of the Json */
  oJsonResponseObject:GetJsonObject("data")
  /* An array of current condition objects - There is only one */
    :GetJsonArray("current_condition")
  /* The first JsonObject in the array */  
    :GetJsonObject(1).

MESSAGE
  "CurrentConditions:" SKIP
  /* Field names are case sensitive */ 
  "  Observation Time:"
  STRING(oCurrentConditionObject:GetCharacter("observation_time")) SKIP
  "  Temperature:"
  STRING(oCurrentConditionObject:GetCharacter("temp_F")) SKIP
  "  Wind Direction:"
  STRING(oCurrentConditionObject:GetCharacter("winddir16Point")) SKIP
  "  Barometric Pressure:"
  STRING(oCurrentConditionObject:GetCharacter("pressure"))
  VIEW-AS ALERT-BOX.


oWeatherArray = oJsonResponseObject
  :GetJsonObject("data")
  :GetJsonArray("weather").

/* To simplify demo, getting weather for 3rd day out. In real program 
   would loop here to get each day's weather */
oWeatherObject = oWeatherArray:GetJsonObject(3).

/* Get the hourly forecast data array */
oHourlyArray = oWeatherObject:GetJsonArray("hourly").
/* Loop through the forecast data. */
DO i = 1 TO oHourlyArray:Length:
  MESSAGE 
    "Hourly Forecast for " STRING(oWeatherObject:GetDate("date")) SKIP
    "  Time:"
    STRING(oHourlyArray:GetJsonObject(i):GetCharacter("time")) SKIP
    
    "  Temp:"  
    STRING(oHourlyArray:GetJsonObject(i):GetCharacter("tempF")) SKIP

    VIEW-AS ALERT-BOX.
END.

MESSAGE "Now that we've shown how to use the JsonObject by parsing it," SKIP
  "here's an entirely different approach using a Progress Dataset."
  VIEW-AS ALERT-BOX.

/* Dynamic DataSet */
CREATE DATASET ProgressDS.
/* If you're lucky, the Json meets the Read-Json requirements */
ProgressDS:READ-JSON("JsonObject", oJsonResponseObject, ?).

/* See what we got back. From this service, we got back 9 tables,
   all our now in our Dataset. Let's loop through and see what their
   names are and what fields we got */
 
DO y = 1 TO ProgressDS:NUM-BUFFERS:
  hBuff = ProgressDS:GET-BUFFER-HANDLE(y).
  DO i = 1 TO hBuff:NUM-FIELDS:
    fieldList = fieldList + hBuff:BUFFER-FIELD(i):Name + " - " +
      hBuff:BUFFER-FIELD(i):data-type + "~n".
  END.
  MESSAGE "Buffer Name: "
    hBuff:NAME SKIP
    hBuff:NUM-FIELDS " Fields" SKIP
    fieldList
    VIEW-AS ALERT-BOX.
  fieldList = "".
END.

/* As a demo, let's show a few values from the first record in the 
   Current_Condition temp-table */
hBuff = ProgressDS:GET-BUFFER-HANDLE("Current_Condition").
hBuff:FIND-FIRST().
MESSAGE 
  "Current Conditions:" SKIP
  "  Time:" hBuff:BUFFER-FIELD("observation_time"):buffer-value SKIP
  "  TempF:" hBuff:BUFFER-FIELD("temp_f"):buffer-value SKIP
  "  Wind:" hBuff:BUFFER-FIELD("WindDir16Point"):buffer-value SKIP
  "  CloudCover:" hBuff:BUFFER-FIELD("cloudcover"):buffer-value
  VIEW-AS ALERT-BOX.



   